@@ -1,9 +1,11 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 module Text.Pandoc.CSS ( foldOrElse
                        , pickStyleAttrProps
                        , pickStylesToKVs
                        )
 where
 
+import Prelude
 import Text.Pandoc.Shared (trim)
 import Text.Parsec
 import Text.Parsec.String
@@ -11,7 +13,7 @@ import Text.Parsec.String
 ruleParser :: Parser (String, String)
 ruleParser = do
     p <- many1 (noneOf ":")  <* char ':'
-    v <- many1 (noneOf ":;") <* (optional $ char ';') <* spaces
+    v <- many1 (noneOf ":;") <* optional (char ';') <* spaces
     return (trim p, trim v)
 
 styleAttrParser :: Parser [(String, String)]
@@ -25,14 +27,14 @@ foldOrElse v xs = foldr (orElse v) v xs
 
 eitherToMaybe :: Either a b -> Maybe b
 eitherToMaybe (Right x) = Just x
-eitherToMaybe _ = Nothing
+eitherToMaybe _         = Nothing
 
 -- | takes a list of keys/properties and a CSS string and
 -- returns the corresponding key-value-pairs.
 pickStylesToKVs :: [String] -> String -> [(String, String)]
 pickStylesToKVs props styleAttr =
   case parse styleAttrParser "" styleAttr of
-    Left _ -> []
+    Left _       -> []
     Right styles -> filter (\s -> fst s `elem` props) styles
 
 -- | takes a list of key/property synonyms and a CSS string and maybe
@@ -40,4 +42,4 @@ pickStylesToKVs props styleAttr =
 pickStyleAttrProps :: [String] -> String -> Maybe String
 pickStyleAttrProps lookupProps styleAttr = do
     styles <- eitherToMaybe $ parse styleAttrParser "" styleAttr
-    foldOrElse Nothing $ map (flip lookup styles) lookupProps
+    foldOrElse Nothing $ map (`lookup` styles) lookupProps
