@@ -1,5 +1,6 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-
-Copyright (C) 2011-2016 John MacFarlane <jgm@berkeley.edu>
+Copyright (C) 2011-2018 John MacFarlane <jgm@berkeley.edu>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -18,7 +19,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module      : Text.Pandoc.MIME
-   Copyright   : Copyright (C) 2011-2016 John MacFarlane
+   Copyright   : Copyright (C) 2011-2018 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -29,11 +30,12 @@ Mime type lookup for ODT writer.
 -}
 module Text.Pandoc.MIME ( MimeType, getMimeType, getMimeTypeDef,
                           extensionFromMimeType )where
-import System.FilePath
-import Data.Char ( toLower )
+import Prelude
+import Data.Char (toLower)
 import Data.List (isPrefixOf, isSuffixOf)
-import Data.Maybe (fromMaybe)
 import qualified Data.Map as M
+import Data.Maybe (fromMaybe)
+import System.FilePath
 
 type MimeType = String
 
@@ -168,10 +170,11 @@ mimeTypesList = -- List borrowed from happstack-server.
            ,("emb","chemical/x-embl-dl-nucleotide")
            ,("embl","chemical/x-embl-dl-nucleotide")
            ,("emf","image/x-emf")
+           ,("emz","application/x-msmetafile")
            ,("eml","message/rfc822")
            ,("ent","chemical/x-ncbi-asn1-ascii")
            ,("eot","application/vnd.ms-fontobject")
-           ,("eps","application/postscript")
+           ,("eps","application/eps")
            ,("etx","text/x-setext")
            ,("exe","application/x-msdos-program")
            ,("ez","application/andrew-inset")
@@ -243,7 +246,7 @@ mimeTypesList = -- List borrowed from happstack-server.
            ,("jpeg","image/jpeg")
            ,("jfif","image/jpeg")
            ,("jpg","image/jpeg")
-           ,("js","application/x-javascript")
+           ,("js","application/javascript")
            ,("kar","audio/midi")
            ,("key","application/pgp-keys")
            ,("kil","application/x-killustrator")
@@ -324,6 +327,7 @@ mimeTypesList = -- List borrowed from happstack-server.
            ,("ogv","video/ogg")
            ,("ogx","application/ogg")
            ,("old","application/x-trash")
+           ,("opus","audio/ogg")
            ,("otg","application/vnd.oasis.opendocument.graphics-template")
            ,("oth","application/vnd.oasis.opendocument.text-web")
            ,("otp","application/vnd.oasis.opendocument.presentation-template")
@@ -524,4 +528,3 @@ mimeTypesList = -- List borrowed from happstack-server.
            ,("zip","application/zip")
            ,("zmt","chemical/x-mopac-input")
            ]
-
