@@ -1,3 +1,4 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE PatternGuards #-}
 
 {-
@@ -32,31 +33,46 @@ Entry point to the odt reader.
 
 module Text.Pandoc.Readers.Odt ( readOdt ) where
 
-import           Codec.Archive.Zip
-import qualified Text.XML.Light                        as XML
+import Prelude
+import Codec.Archive.Zip
+import qualified Text.XML.Light as XML
 
-import qualified Data.ByteString.Lazy                  as B
+import qualified Data.ByteString.Lazy as B
 
-import           System.FilePath
+import System.FilePath
 
-import           Text.Pandoc.Definition
-import           Text.Pandoc.Error
-import           Text.Pandoc.Options
-import           Text.Pandoc.MediaBag
-import qualified Text.Pandoc.UTF8                      as UTF8
+import Control.Monad.Except (throwError)
 
-import           Text.Pandoc.Readers.Odt.ContentReader
-import           Text.Pandoc.Readers.Odt.StyleReader
+import Text.Pandoc.Class (PandocMonad)
+import qualified Text.Pandoc.Class as P
+import Text.Pandoc.Definition
+import Text.Pandoc.Error
+import Text.Pandoc.MediaBag
+import Text.Pandoc.Options
+import qualified Text.Pandoc.UTF8 as UTF8
 
-import           Text.Pandoc.Readers.Odt.Generic.XMLConverter
-import           Text.Pandoc.Readers.Odt.Generic.Fallible
-import           Text.Pandoc.Shared (filteredFilesFromArchive)
+import Text.Pandoc.Readers.Odt.ContentReader
+import Text.Pandoc.Readers.Odt.StyleReader
+
+import Text.Pandoc.Readers.Odt.Generic.Fallible
+import Text.Pandoc.Readers.Odt.Generic.XMLConverter
+import Text.Pandoc.Shared (filteredFilesFromArchive)
+
+readOdt :: PandocMonad m
+        => ReaderOptions
+        -> B.ByteString
+        -> m Pandoc
+readOdt opts bytes = case readOdt' opts bytes of
+  Right (doc, mb) -> do
+    P.setMediaBag mb
+    return doc
+  Left e -> throwError e
 
 --
-readOdt :: ReaderOptions
-        -> B.ByteString
-        -> Either PandocError (Pandoc, MediaBag)
-readOdt _ bytes = bytesToOdt bytes-- of
+readOdt' :: ReaderOptions
+         -> B.ByteString
+         -> Either PandocError (Pandoc, MediaBag)
+readOdt' _ bytes = bytesToOdt bytes-- of
 --                    Right (pandoc, mediaBag) -> Right (pandoc , mediaBag)
 --                    Left  err                -> Left err
 
@@ -64,7 +80,7 @@ readOdt _ bytes = bytesToOdt bytes-- of
 bytesToOdt :: B.ByteString -> Either PandocError (Pandoc, MediaBag)
 bytesToOdt bytes = case toArchiveOrFail bytes of
   Right archive -> archiveToOdt archive
-  Left _        -> Left $ ParseFailure "Couldn't parse odt file."
+  Left _        -> Left $ PandocParseError "Couldn't parse odt file."
 
 --
 archiveToOdt :: Archive -> Either PandocError (Pandoc, MediaBag)
@@ -85,7 +101,7 @@ archiveToOdt archive
 
   | otherwise
     -- Not very detailed, but I don't think more information would be helpful
-  = Left $ ParseFailure "Couldn't parse odt file."
+  = Left $ PandocParseError "Couldn't parse odt file."
     where
       filePathIsOdtMedia :: FilePath -> Bool
       filePathIsOdtMedia fp =
