@@ -1,5 +1,6 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-
-Copyright (C) 2014-2016 Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
+Copyright (C) 2014-2018 Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -17,8 +18,8 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 -}
 
 {- |
-   Module      : Text.Pandoc.Readers.Org.Options
-   Copyright   : Copyright (C) 2014-2016 Albert Krewinkel
+   Module      : Text.Pandoc.Readers.Org.BlockStarts
+   Copyright   : Copyright (C) 2014-2018 Albert Krewinkel
    License     : GNU GPL, version 2 or above
 
    Maintainer  : Albert Krewinkel <tarleb+pandoc@moltkeplatz.de>
@@ -40,11 +41,12 @@ module Text.Pandoc.Readers.Org.BlockStarts
   , endOfBlock
   ) where
 
-import Control.Monad ( void )
+import Prelude
+import Control.Monad (void)
 import Text.Pandoc.Readers.Org.Parsing
 
 -- | Horizontal Line (five -- dashes or more)
-hline :: OrgParser ()
+hline :: Monad m => OrgParser m ()
 hline = try $ do
   skipSpaces
   string "-----"
@@ -54,58 +56,66 @@ hline = try $ do
   return ()
 
 -- | Read the start of a header line, return the header level
-headerStart :: OrgParser Int
+headerStart :: Monad m => OrgParser m Int
 headerStart = try $
   (length <$> many1 (char '*')) <* many1 (char ' ') <* updateLastPreCharPos
 
-tableStart :: OrgParser Char
+tableStart :: Monad m => OrgParser m Char
 tableStart = try $ skipSpaces *> char '|'
 
-latexEnvStart :: OrgParser String
-latexEnvStart = try $ do
+gridTableStart :: Monad m => OrgParser m ()
+gridTableStart = try $ skipSpaces <* char '+' <* char '-'
+
+
+latexEnvStart :: Monad m => OrgParser m String
+latexEnvStart = try $
   skipSpaces *> string "\\begin{"
              *> latexEnvName
              <* string "}"
              <* blankline
  where
-   latexEnvName :: OrgParser String
+   latexEnvName :: Monad m => OrgParser m String
    latexEnvName = try $ mappend <$> many1 alphaNum <*> option "" (string "*")
 
+bulletListStart :: Monad m => OrgParser m Int
+bulletListStart = try $ do
+  ind <- length <$> many spaceChar
+   -- Unindented lists cannot use '*' bullets.
+  oneOf (if ind == 0 then "+-" else "*+-")
+  skipSpaces1 <|> lookAhead eol
+  return (ind + 1)
 
--- | Parses bullet list marker.
-bulletListStart :: OrgParser ()
-bulletListStart = try $
-  choice
-  [ () <$ skipSpaces  <* oneOf "+-" <* skipSpaces1
-  , () <$ skipSpaces1 <* char '*'   <* skipSpaces1
-  ]
+genericListStart :: Monad m
+                 => OrgParser m String
+                 -> OrgParser m Int
+genericListStart listMarker = try $ do
+  ind <- length <$> many spaceChar
+  void listMarker
+  skipSpaces1 <|> lookAhead eol
+  return (ind + 1)
 
-genericListStart :: OrgParser String
-                 -> OrgParser Int
-genericListStart listMarker = try $
-  (+) <$> (length <$> many spaceChar)
-      <*> (length <$> listMarker <* many1 spaceChar)
+eol :: Monad m => OrgParser m ()
+eol = void (char '\n')
 
-orderedListStart :: OrgParser Int
+orderedListStart :: Monad m => OrgParser m Int
 orderedListStart = genericListStart orderedListMarker
   -- Ordered list markers allowed in org-mode
   where orderedListMarker = mappend <$> many1 digit <*> (pure <$> oneOf ".)")
 
-drawerStart :: OrgParser String
-drawerStart = try $
-  skipSpaces *> drawerName <* skipSpaces <* newline
+drawerStart :: Monad m => OrgParser m String
+drawerStart = try $ skipSpaces *> drawerName <* skipSpaces <* newline
  where drawerName = char ':' *> manyTill nonspaceChar (char ':')
 
-metaLineStart :: OrgParser ()
+metaLineStart :: Monad m => OrgParser m ()
 metaLineStart = try $ skipSpaces <* string "#+"
 
-commentLineStart :: OrgParser ()
+commentLineStart :: Monad m => OrgParser m ()
 commentLineStart = try $ skipSpaces <* string "# "
 
-exampleLineStart :: OrgParser ()
+exampleLineStart :: Monad m => OrgParser m ()
 exampleLineStart = () <$ try (skipSpaces *> string ": ")
 
-noteMarker :: OrgParser String
+noteMarker :: Monad m => OrgParser m String
 noteMarker = try $ do
   char '['
   choice [ many1Till digit (char ']')
@@ -114,17 +124,18 @@ noteMarker = try $ do
          ]
 
 -- | Succeeds if the parser is at the end of a block.
-endOfBlock :: OrgParser ()
-endOfBlock = lookAhead . try $ do
-    void blankline <|> anyBlockStart
+endOfBlock :: Monad m => OrgParser m ()
+endOfBlock = lookAhead . try $
+  void blankline <|> anyBlockStart
  where
    -- Succeeds if there is a new block starting at this position.
-   anyBlockStart :: OrgParser ()
+   anyBlockStart :: Monad m => OrgParser m ()
    anyBlockStart = try . choice $
      [ exampleLineStart
      , hline
      , metaLineStart
      , commentLineStart
+     , gridTableStart
      , void noteMarker
      , void tableStart
      , void drawerStart
@@ -133,4 +144,3 @@ endOfBlock = lookAhead . try $ do
      , void bulletListStart
      , void orderedListStart
      ]
-
