@@ -1,6 +1,7 @@
+{-# LANGUAGE NoImplicitPrelude #-}
 {-# LANGUAGE OverloadedStrings #-}
 {-
-Copyright (C) 2006-2015 John MacFarlane <jgm@berkeley.edu>
+Copyright (C) 2006-2018 John MacFarlane <jgm@berkeley.edu>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
@@ -19,7 +20,7 @@ Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 {- |
    Module      : Text.Pandoc.Writers.Native
-   Copyright   : Copyright (C) 2006-2015 John MacFarlane
+   Copyright   : Copyright (C) 2006-2018 John MacFarlane
    License     : GNU GPL, version 2 or above
 
    Maintainer  : John MacFarlane <jgm@berkeley.edu>
@@ -30,14 +31,18 @@ Conversion of a 'Pandoc' document to a string representation.
 -}
 module Text.Pandoc.Writers.Native ( writeNative )
 where
-import Text.Pandoc.Options ( WriterOptions(..), WrapOption(..) )
-import Data.List ( intersperse )
+import Prelude
+import Data.List (intersperse)
+import Data.Text (Text)
+import Text.Pandoc.Class (PandocMonad)
 import Text.Pandoc.Definition
+import Text.Pandoc.Options (WrapOption (..), WriterOptions (..))
 import Text.Pandoc.Pretty
 
 prettyList :: [Doc] -> Doc
 prettyList ds =
-  "[" <> (cat $ intersperse (cr <> ",") $ map (nest 1) ds) <> "]"
+  "[" <>
+  cat (intersperse (cr <> ",") $ map (nest 1) ds) <> "]"
 
 -- | Prettyprint Pandoc block element.
 prettyBlock :: Block -> Doc
@@ -47,12 +52,12 @@ prettyBlock (BlockQuote blocks) =
   "BlockQuote" $$ prettyList (map prettyBlock blocks)
 prettyBlock (OrderedList attribs blockLists) =
   "OrderedList" <> space <> text (show attribs) $$
-  (prettyList $ map (prettyList . map prettyBlock) blockLists)
+  prettyList (map (prettyList . map prettyBlock) blockLists)
 prettyBlock (BulletList blockLists) =
   "BulletList" $$
-  (prettyList $ map (prettyList . map prettyBlock) blockLists)
+  prettyList (map (prettyList . map prettyBlock) blockLists)
 prettyBlock (DefinitionList items) = "DefinitionList" $$
-  (prettyList $ map deflistitem items)
+  prettyList (map deflistitem items)
     where deflistitem (term, defs) = "(" <> text (show term) <> "," <> cr <>
            nest 1 (prettyList $ map (prettyList . map prettyBlock) defs) <> ")"
 prettyBlock (Table caption aligns widths header rows) =
@@ -66,8 +71,8 @@ prettyBlock (Div attr blocks) =
 prettyBlock block = text $ show block
 
 -- | Prettyprint Pandoc document.
-writeNative :: WriterOptions -> Pandoc -> String
-writeNative opts (Pandoc meta blocks) =
+writeNative :: PandocMonad m => WriterOptions -> Pandoc -> m Text
+writeNative opts (Pandoc meta blocks) = return $
   let colwidth = if writerWrapText opts == WrapAuto
                     then Just $ writerColumns opts
                     else Nothing
