@@ -35,7 +35,7 @@ module Text.Pandoc.Writers.RST ( writeRST, flatten ) where
 import Prelude
 import Control.Monad.State.Strict
 import Data.Char (isSpace, toLower)
-import Data.List (isPrefixOf, stripPrefix)
+import Data.List (isPrefixOf, stripPrefix, transpose)
 import Data.Maybe (fromMaybe)
 import Data.Text (Text, stripEnd)
 import qualified Text.Pandoc.Builder as B
@@ -82,14 +82,12 @@ pandocToRST (Pandoc meta blocks) = do
                     else Nothing
   let render' :: Doc -> Text
       render' = render colwidth
-  let subtit = case lookupMeta "subtitle" meta of
-                    Just (MetaBlocks [Plain xs]) -> xs
-                    _                            -> []
+  let subtit = lookupMetaInlines "subtitle" meta
   title <- titleToRST (docTitle meta) subtit
   metadata <- metaToJSON opts
                 (fmap render' . blockListToRST)
                 (fmap (stripEnd . render') . inlineListToRST)
-                $ B.deleteMeta "title" $ B.deleteMeta "subtitle" meta
+                meta
   body <- blockListToRST' True $ case writerTemplate opts of
                                       Just _  -> normalizeHeadings 1 blocks
                                       Nothing -> blocks
@@ -103,8 +101,9 @@ pandocToRST (Pandoc meta blocks) = do
   let context = defField "body" main
               $ defField "toc" (writerTableOfContents opts)
               $ defField "toc-depth" (show $ writerTOCDepth opts)
+              $ defField "number-sections" (writerNumberSections opts)
               $ defField "math" hasMath
-              $ defField "title" (render Nothing title :: String)
+              $ defField "titleblock" (render Nothing title :: String)
               $ defField "math" hasMath
               $ defField "rawtex" rawTeX metadata
   case writerTemplate opts of
@@ -209,11 +208,26 @@ blockToRST :: PandocMonad m
            => Block         -- ^ Block element
            -> RST m Doc
 blockToRST Null = return empty
-blockToRST (Div attr bs) = do
+blockToRST (Div ("",["admonition-title"],[]) _) = return empty
+  -- this is generated by the rst reader and can safely be
+  -- omitted when we're generating rst
+blockToRST (Div (ident,classes,_kvs) bs) = do
   contents <- blockListToRST bs
-  let startTag = ".. raw:: html" $+$ nest 3 (tagWithAttrs "div" attr)
-  let endTag = ".. raw:: html" $+$ nest 3 "</div>"
-  return $ blankline <> startTag $+$ contents $+$ endTag $$ blankline
+  let admonitions = ["attention","caution","danger","error","hint",
+                     "important","note","tip","warning","admonition"]
+  let admonition = case classes of
+                        (cl:_)
+                          | cl `elem` admonitions
+                          -> ".. " <> text cl <> "::"
+                        cls -> ".. container::" <> space <>
+                                   text (unwords (filter (/= "container") cls))
+  return $ blankline $$
+           admonition $$
+           (if null ident
+               then blankline
+               else "   :name: " <> text ident $$ blankline) $$
+           nest 3 contents $$
+           blankline
 blockToRST (Plain inlines) = inlineListToRST inlines
 -- title beginning with fig: indicates that the image is a figure
 blockToRST (Para [Image attr txt (src,'f':'i':'g':':':tit)]) = do
@@ -236,6 +250,7 @@ blockToRST (LineBlock lns) =
   linesToLineBlock lns
 blockToRST (RawBlock f@(Format f') str)
   | f == "rst" = return $ text str
+  | f == "tex" = blockToRST (RawBlock (Format "latex") str)
   | otherwise  = return $ blankline <> ".. raw:: " <>
                     text (map toLower f') $+$
                     nest 3 (text str) $$ blankline
@@ -245,7 +260,8 @@ blockToRST (Header level (name,classes,_) inlines) = do
   contents <- inlineListToRST inlines
   -- we calculate the id that would be used by auto_identifiers
   -- so we know whether to print an explicit identifier
-  let autoId = uniqueIdent inlines mempty
+  opts <- gets stOptions
+  let autoId = uniqueIdent (writerExtensions opts) inlines mempty
   isTopLevel <- gets stTopLevel
   if isTopLevel
     then do
@@ -272,7 +288,8 @@ blockToRST (CodeBlock (_,classes,kvs) str) = do
      then return $ prefixed "> " (text str) $$ blankline
      else return $
           (case [c | c <- classes,
-                     c `notElem` ["sourceCode","literate","numberLines"]] of
+                     c `notElem` ["sourceCode","literate","numberLines",
+                                  "number-lines","example"]] of
              []       -> "::"
              (lang:_) -> (".. code:: " <> text lang) $$ numberlines)
           $+$ nest 3 (text str) $$ blankline
@@ -288,9 +305,12 @@ blockToRST (Table caption aligns widths headers rows) = do
          modify $ \st -> st{ stOptions = oldOpts }
          return result
   opts <- gets stOptions
-  tbl <- gridTable opts blocksToDoc (all null headers)
-            (map (const AlignDefault) aligns) widths
-            headers rows
+  let isSimple = all (== 0) widths
+  tbl <- if isSimple
+            then simpleTable opts blocksToDoc headers rows
+            else gridTable opts blocksToDoc (all null headers)
+                  (map (const AlignDefault) aligns) widths
+                  headers rows
   return $ if null caption
               then tbl $$ blankline
               else (".. table:: " <> caption') $$ blankline $$ nest 3 tbl $$
@@ -331,7 +351,7 @@ orderedListItemToRST marker items = do
   let marker' = marker ++ " "
   return $ hang (length marker') (text marker') $ contents <> cr
 
--- | Convert defintion list item (label, list of blocks) to RST.
+-- | Convert definition list item (label, list of blocks) to RST.
 definitionListItemToRST :: PandocMonad m => ([Inline], [[Block]]) -> RST m Doc
 definitionListItemToRST (label, defs) = do
   label' <- inlineListToRST label
@@ -470,6 +490,8 @@ flatten outer
           -- them and they will be readable and parsable
           (Quoted _ _, _)          -> keep f i
           (_, Quoted _ _)          -> keep f i
+          -- inlineToRST handles this case properly so it's safe to keep
+          (Link _ _ _, Image _ _ _) -> keep f i
           -- parent inlines would prevent links from being correctly
           -- parsed, in this case we prioritise the content over the
           -- style
@@ -569,15 +591,18 @@ inlineToRST (Quoted DoubleQuote lst) = do
      else return $ "“" <> contents <> "”"
 inlineToRST (Cite _  lst) =
   writeInlines lst
+inlineToRST (Code (_,["interpreted-text"],[("role",role)]) str) = do
+  return $ ":" <> text role <> ":`" <> text str <> "`"
 inlineToRST (Code _ str) = do
   opts <- gets stOptions
   -- we trim the string because the delimiters must adjoin a
   -- non-space character; see #3496
   -- we use :literal: when the code contains backticks, since
   -- :literal: allows backslash-escapes; see #3974
-  return $ if '`' `elem` str
-              then ":literal:`" <> text (escapeString opts (trim str)) <> "`"
-              else "``" <> text (trim str) <> "``"
+  return $
+    if '`' `elem` str
+       then ":literal:`" <> text (escapeString opts (trim str)) <> "`"
+       else "``" <> text (trim str) <> "``"
 inlineToRST (Str str) = do
   opts <- gets stOptions
   return $ text $
@@ -672,3 +697,30 @@ imageDimsToRST attr = do
                           Just dim -> cols dim
                           Nothing  -> empty
   return $ cr <> name $$ showDim Width $$ showDim Height
+
+simpleTable :: PandocMonad m
+            => WriterOptions
+            -> (WriterOptions -> [Block] -> m Doc)
+            -> [[Block]]
+            -> [[[Block]]]
+            -> m Doc
+simpleTable opts blocksToDoc headers rows = do
+  -- can't have empty cells in first column:
+  let fixEmpties (d:ds) = if isEmpty d
+                             then text "\\ " : ds
+                             else d : ds
+      fixEmpties [] = []
+  headerDocs <- if all null headers
+                   then return []
+                   else fixEmpties <$> mapM (blocksToDoc opts) headers
+  rowDocs <- mapM (fmap fixEmpties . mapM (blocksToDoc opts)) rows
+  let numChars [] = 0
+      numChars xs = maximum . map offset $ xs
+  let colWidths = map numChars $ transpose (headerDocs : rowDocs)
+  let toRow = hsep . zipWith lblock colWidths
+  let hline = hsep (map (\n -> text (replicate n '=')) colWidths)
+  let hdr = if all null headers
+               then mempty
+               else hline $$ toRow headerDocs
+  let bdy = vcat $ map toRow rowDocs
+  return $ hdr $$ hline $$ bdy $$ hline
